export { default as Sequence } from './Sequence';
declare type F<T> = {
    [key in keyof T]: {
        success: key;
        error: 'error';
        loading: 'loading';
    };
};
declare type B<T> = {
    [key in keyof T]: key;
};
export declare const NO_ERROR_TYPES = -1;
export declare function composeTypes<T1, T2>(config: {
    prefix: string;
    BasicTypes: T1;
    FetchTypes: T2;
}): B<T1> & F<T2>;
export declare type ActionCreator<T> = <S>(type: S) => (params?: T) => {
    type: S;
    payload: T;
};
export declare function createAction<T>(type: T, stateKey?: string): <P, R = P>(fn?: (params?: P, ...args: any[]) => R) => (params?: P, ...args: any[]) => {
    type: T;
    stateKey: string;
    payload: R;
} | {
    type: T;
    payload: R;
    stateKey?: undefined;
};
interface IFetchTypes<key> {
    error: 'error';
    success: key;
    loading: 'loading';
}
export declare enum Method {
    Get = "GET",
    Post = "POST",
    Put = "PUT",
    Delete = "DELETE"
}
export declare function createFetchAction<Key>(types: IFetchTypes<Key>, url: string, method?: Method): <Params, Response>(stateKey?: string) => (params?: Params, meta?: any) => {
    stateKey: string;
    types: IFetchTypes<Key>;
    meta: any;
    params: Params;
    url: string;
    method: Method;
} & {
    type: Key;
    payload: Response;
} & Partial<Promise<Response>>;
export declare type ReturnState<ReducerMap> = {
    [key in keyof ReducerMap]: ReducerMap[key] extends (state: any, action: any) => infer R ? R : any;
};
declare type ValueOf<T> = T[keyof T];
export declare type ActionType<Actions> = ValueOf<{
    [key in keyof Actions]: Actions[key] extends (...args: any[]) => infer R ? R : never;
}> | ValueOf<{
    [key in keyof Actions]: Actions[key] extends (arg: never, ...args: any[]) => infer R ? R : never;
}> | {
    type: 'error';
    payload?: {
        message: string;
        [key: string]: any;
    };
    params?: any;
    meta?: any;
} | {
    type: 'loading';
    payload?: any;
    params?: any;
    meta?: any;
};
export declare function getMergedDefinetion<Result>(map: (state: any) => Result): Result;
export declare type Dispatch = <Payload>(action: {
    type?: string;
    payload?: Payload;
}) => Promise<Payload>;
export declare type ThunkAction<Payload = never> = {
    type?: '@thunkAction';
    payload?: never;
    (dispatch: Dispatch, getState?: any): any;
} & Partial<Promise<Payload>>;
export declare class AsyncTuple<T> {
    loading: boolean;
    error: boolean;
    message?: string;
    data?: T;
    [x: string]: any;
    constructor(initData: T);
    constructor(initLoading: boolean, initData: T);
    static handleLoading<K extends keyof T, T extends Object>(stateKey: K, state: T, extraProps?: Object): T;
    static handleSuccess<K extends keyof T, T extends Object>(stateKey: K, state: T, action: any, extraProps?: Object): T;
    static handleError<K extends keyof T, T extends Object>(stateKey: K, state: T, action: any, extraProps?: Object): T;
    static defaultProcess<K extends keyof T, T extends Object>(stateKey: K, state: T, action: any, fetchType: 'loading' | 'success' | 'error' | 'normal'): T;
    static handleAll<State>(prefix: string, state: State, action: any, process?: typeof AsyncTuple.defaultProcess): State;
}
export declare function safeGet<T, K1 extends keyof T>(data: T, keys: [K1], defaultValue?: any): T[K1];
export declare function safeGet<T, K1 extends keyof T, K2 extends keyof T[K1]>(data: T, keys: [K1, K2], defaultValue?: any): T[K1][K2];
export declare function safeGet<T, K1 extends keyof T, K2 extends keyof T[K1], K3 extends keyof T[K1][K2]>(data: T, keys: [K1, K2, K3], defaultValue?: any): T[K1][K2][K3];
export declare function safeGet<T, K1 extends keyof T, K2 extends keyof T[K1], K3 extends keyof T[K1][K2], K4 extends keyof T[K1][K2][K3]>(data: T, keys: [K1, K2, K3, K4], defaultValue?: any): T[K1][K2][K3][K4];
export declare function safeGet<T, K1 extends keyof T, K2 extends keyof T[K1], K3 extends keyof T[K1][K2], K4 extends keyof T[K1][K2][K3], K5 extends keyof T[K1][K2][K3][K4]>(data: T, keys: [K1, K2, K3, K4, K5], defaultValue?: any): T[K1][K2][K3][K4][K5];
export declare function safeGet<T, K1 extends keyof T, K2 extends keyof T[K1], K3 extends keyof T[K1][K2], K4 extends keyof T[K1][K2][K3], K5 extends keyof T[K1][K2][K3][K4], K6 extends keyof T[K1][K2][K3][K4][K5]>(data: T, keys: [K1, K2, K3, K4, K5, K6], defaultValue?: any): T[K1][K2][K3][K4][K5][K6];
